/*******************************************************************************
  main program
*******************************************************************************/
program define welfare_est, eclass

	/***************************************************************************
	  set Stata version
	***************************************************************************/
	version 16.1 
	
	tempfile user_data
	quietly save `user_data'
	
	/***************************************************************************
	  define program syntax
	***************************************************************************/
	
	syntax varlist(numeric min = 3) [if] [in], iv(string) [fcest(string) wcis(string) figure(string)]
	
	/***************************************************************************
	  parse input
	***************************************************************************/

	/*mark the sample*/
	tempvar touse
	mark `touse' `if' `in'
	
	/*parse passed in variable list*/
	tokenize `varlist'
	local d `1'
	local vc `2'
	local r `3'
	macro shift 3
	local controls `*'
	
	/*get instrumental variables from that option*/ 
	local iv `iv'
	
	/*parse welfare loss confidence interval options*/
	if "`wcis'" != ""{	
		parse_wcis_opt `wcis'
		local bs_switch = "`s(wcis_switch)'"
		local reps = `s(wcis_reps)'
		local seed = `s(wcis_seed)'
		local level = `s(wcis_level)'
		local level_tail = (100 - `level')/2
		local ll_pctile = `level_tail'
		local ul_pctile = 100 - `level_tail'
	}

	else if "`wcis'" == ""{
		local bs_switch = "off"	
	}
	
	/*parse fixed cost estimation options*/
	if "`fcest'" != ""{	
		parse_fcest_opt `fcest'
		local fcest_switch = "`s(fcest_switch)'"
		local fc_input = `s(fcest_fc_input)'
		local grid = "`s(fcest_grid)'"
		local rtarget = "`s(fcest_rtarget)'"
		local qtarget = "`s(fcest_qtarget)'"
	}

	else if "`fcest'" == ""{
		local fcest_switch = "off"
		local fc_input = 0
	}
	
	/*parse figure appearance options*/
	if "`figure'" != ""{	
		parse_figure_opt `figure'
		local figure_switch = "`s(figure_switch)'"
		local scheme = "`s(figure_scheme)'"
		local d_wtp = "`s(figure_d_wtp)'"
		local ac_mc = "`s(figure_ac_mc)'"
		local dwl = "`s(figure_dwl)'"
	}
	
	
	/*make sure arguments are valid*/
	capture assert inlist(`d', 0, 1) 
	if _rc != 0{
		
		display as error "All quantity observations must be binary."
		exit 198
		
	}
	
	capture assert `r' >= 0
	if _rc != 0 {
		
		display as error "All interest rates must be positive."
		exit 198
		
	}
		
	/***************************************************************************
	  housekeeping
	***************************************************************************/
	
	/*demean all control variables*/
	local controls_demean ""
	if "`controls'" != ""{
		quietly foreach var of varlist `controls'{
			sum `var'
			tempvar `var'_demean
			gen ``var'_demean' = `var' - r(mean)
			local controls_demean `controls_demean' ``var'_demean'
		}
	}
	
	/***************************************************************************
	 determine the fixed cost to be used in the estimation
	***************************************************************************/
	
	/*if not estimating fixed cost, use user calibrated input */
	if "`fcest_switch'" == "off"{
		local fc_star = `fc_input'
	}	
	
	/*otherwise, estimate the fixed cost*/
	else if "`fcest_switch'" == "on"{
			
		/*temporary variable to store total cost at each grid point*/
		tempvar tc
		qui gen `tc' = .

		/*matrix to store objective function values at each grid point*/
		tempname grid_eval
		local nrows : word count `grid'
		matrix `grid_eval' = J(`nrows', 4, .)
		matrix coln `grid_eval' = "fc_candidate" "QEQ" "rEQ" "obj_fnt"
		
		/*calcluate equilbirum outcome and objective function at each grid point*/
		local row = 0
		foreach x in `grid'{
				
				local ++row
				qui replace `tc' = `vc' + `x' 

				/*run demand regressions and store coefficients*/		
				qui ivregress 2sls `d' `controls_demean' (`r' = `iv') if `touse'
				local alpha_d = _b[_cons]
				local beta_d  = _b[`r']	
						
				/*run average cost regressions and store coefficients*/
				qui ivregress 2sls `tc' `controls_demean' (`r' = `iv') if `touse'				
				local alpha_c = _b[_cons]
				local beta_c  = _b[`r']
						
				/*calculate some intermediate quantities*/						
				local phi_c = 1 + `x' - `alpha_c' - `beta_c'
				local psi_c = `phi_c'^2 - 4*`alpha_c'*`beta_c'
						
				/*calculate equilibrium outcomes*/					
				local QEQ = `alpha_d' + (`beta_d'*(`phi_c' - sqrt(`psi_c')))/(2*`beta_c')
				local rEQ = ((`phi_c' - sqrt(`psi_c')))/(2*`beta_c')
				
				/*calculate the objective function value*/
				local obj_fnt = (`rtarget'-`rEQ')^2 + (`qtarget'-`QEQ')^2
				
				/*store results*/
				matrix `grid_eval'[`row',1] = `x'
				matrix `grid_eval'[`row',2] = `QEQ'
				matrix `grid_eval'[`row',3] = `rEQ'
				matrix `grid_eval'[`row',4] = `obj_fnt'
		
		}

		/*estimated fixed cost is value that minimizes the objective function*/
		preserve
			clear
			svmat `grid_eval', names(col)
			sort obj_fnt 
			local fc_star = fc_candidate[1]
		restore
		
	}
	
	/*add fixed cost to variable cost*/
	tempvar c
	quietly: gen `c' = `vc' + `fc_star'
	
	/***************************************************************************
	  estimate demand curve
	***************************************************************************/
	
	/*temporary matrix names for storing estimates*/
	tempname b_d_fs b_d_iv V_d_fs V_d_iv
	
	/*iv*/
	quietly{
		
		if "`bs_switch'" == "off"{
			
			/*run regression*/
			ivregress 2sls `d' `controls_demean' (`r' = `iv') if `touse'
			
			/*store estimates*/
			matrix `b_d_iv' = e(b)
			matrix `V_d_iv' = e(V)	
		}	
		
		else if "`bs_switch'" == "on"{
			
			/*assign temporary file name to store bootstrap replicates*/
			tempfile bs_d
			
			/*run regression*/
			bootstrap, reps(`reps') seed(`seed') saving(`bs_d', double): ///
				ivregress 2sls `d' `controls_demean' (`r' = `iv') if `touse'
			
			/*store estimates*/
			matrix `b_d_iv' = e(b)
			matrix `V_d_iv' = e(V_modelbased)	
		}
	}
	
	local alpha_d = _b[_cons]
	local beta_d   = _b[`r']
	local N_d = e(N)
	
	matrix coleq `b_d_iv' = demand_iv
	matrix coleq `V_d_iv' = demand_iv
	matrix roweq `V_d_iv' = demand_iv			
		
	matrix coln `b_d_iv' = `r' `controls' _cons
	matrix coln `V_d_iv' = `r' `controls' _cons
	matrix rown `V_d_iv' = `r' `controls' _cons
		
	/*first stage*/	
	quietly: reg `r' `iv' `controls_demean' if e(sample)
	quietly: test `iv'
	local F_d = r(F)
	matrix `b_d_fs' = e(b)
	matrix `V_d_fs' = e(V)
		
	matrix coleq `b_d_fs' = demand_fs
	matrix coleq `V_d_fs' = demand_fs
	matrix roweq `V_d_fs' = demand_fs	
		
	matrix coln `b_d_fs' = `iv' `controls' _cons
	matrix coln `V_d_fs' = `iv' `controls' _cons
	matrix rown `V_d_fs' = `iv' `controls' _cons
	
	/***************************************************************************
	  estimate cost curve
	***************************************************************************/	
	
	/*temporary matrix names for storing estimates*/
	tempname b_c_fs V_c_fs b_c_iv V_c_iv bs_c
	
	/*iv*/
	quietly{
		
		if "`bs_switch'" == "off"{
			
			/*run regression*/
			ivregress 2sls `c' `controls_demean' (`r' = `iv') if `touse'
			
			/*store estimates*/
			matrix `b_c_iv' = e(b)
			matrix `V_c_iv' = e(V)
		}			
		
		else if "`bs_switch'" == "on"{
			
			/*assign temporary file name to store bootstrap replicates*/
			tempfile bs_c
			
			/*run regression*/
			bootstrap, reps(`reps') seed(`seed') saving(`bs_c', double): ///
				ivregress 2sls `c' `controls_demean' (`r' = `iv') if `touse'
				
			/*store estimates*/	
			matrix `b_c_iv' = e(b)
			matrix `V_c_iv' = e(V_modelbased)
		}
	}
		
	local alpha_c = _b[_cons]
	local beta_c   = _b[`r']
	local N_c = e(N)
		
	matrix coleq `b_c_iv' = cost_iv
	matrix coleq `V_c_iv' = cost_iv
	matrix roweq `V_c_iv' = cost_iv			
		
	matrix coln `b_c_iv' = `r' `controls' _cons
	matrix coln `V_c_iv' = `r' `controls' _cons
	matrix rown `V_c_iv' = `r' `controls' _cons	
		
	/*first stage*/	
	quietly: reg `r' `iv' `controls_demean' if e(sample)
	quietly: test `iv'
	local F_c = r(F)
	matrix `b_c_fs' = e(b)
	matrix `V_c_fs' = e(V)
		
	matrix coleq `b_c_fs' = cost_fs
	matrix coleq `V_c_fs' = cost_fs
	matrix roweq `V_c_fs' = cost_fs		
		
	matrix coln `b_c_fs' = `iv' `controls' _cons
	matrix coln `V_c_fs' = `iv' `controls' _cons
	matrix rown `V_c_fs' = `iv' `controls' _cons

	/***************************************************************************
	  calculate implied welfare losses
	***************************************************************************/
	
	/*temporary matrix name for storing estimates*/
	tempname b_w
	
	/*calculate some intermediate quantities*/
	local alpha_m = `alpha_d'*`beta_c'/`beta_d' + `alpha_c'
	local beta_m  = 2*`beta_c'
	
	local phi_c = 1 + `fc_star' - `alpha_c' - `beta_c'
	local psi_c = `phi_c'^2 - 4*`alpha_c'*`beta_c'
	
	local phi_m = 1 + `fc_star' - `alpha_m' - `beta_m'
	local psi_m = `phi_m'^2 - 4*`alpha_m'*`beta_m'
	
	/*calculate efficient and equilibrium outcomes*/
	local QEF = `alpha_d' + (`beta_d'*(`phi_m' - sqrt(`psi_m')))/(2*`beta_m')
	local rEF = ((`phi_m' - sqrt(`psi_m')))/(2*`beta_m')
	
	local QEQ = `alpha_d' + (`beta_d'*(`phi_c' - sqrt(`psi_c')))/(2*`beta_c')
	local rEQ = ((`phi_c' - sqrt(`psi_c')))/(2*`beta_c')
	
	/*approximate welfare loss*/
	local MC  = `alpha_m'+`beta_m'*`rEQ'
	local WTP = `rEQ'*(1+`fc_star'-`alpha_m'-`beta_m'*`rEQ')
	local DWL_approx = 50*(`QEF'-`QEQ')*(`WTP'-`MC')
	
	/*exact welfare loss*/
	tempvar x r_d mc wtp fntDemand fntMC
	quietly{
		range `x' `QEQ' `QEF' 100
		gen `r_d' = (`x'-`alpha_d')/`beta_d'
		gen `mc'  = `alpha_m'+`beta_m'*`r_d'
		gen `wtp' = (1+`fc_star'-`mc')*`r_d'
		integ `wtp' `x', gen(`fntDemand')
		local IntDemand = r(integral)
		integ `mc'  `x', gen(`fntMC')
		local IntMC = r(integral)
		local DWL_exact = 100*(`IntDemand'-`IntMC')
	}
	
	/*store estimates*/
	matrix `b_w' = `rEQ', `QEQ', `rEF', `QEF', `DWL_approx', `DWL_exact'
	matrix coln `b_w' = rEQ QEQ rEF QEF DWL_approx DWL_exact
	matrix rown `b_w' = "estimate"
	
	/***************************************************************************
	  calculate bootstrapped confidence intervals for implied welfare losses
	***************************************************************************/	
	
	if "`bs_switch'" == "on"{
	
	quietly{
	preserve
	
		/*temporary matrix name for storing confidence intervals*/
		tempname ci_w
		
		/*temp var names for demand/cost parameters in each replicate*/
		tempvar alpha_d_r beta_d_r alpha_c_r beta_c_r
		
		/*temp var names for intermediate quantities in each replicate*/
		tempvar alpha_m_r beta_m_r phi_c_r psi_c_r phi_m_r psi_m_r

		/*temp var names for efficient/equilibrium outcome in each replicate*/
		tempvar QEF_r rEF_r QEQ_r rEQ_r

		/*temp var names for welfare loss estimates in each replicate*/
		tempvar MC_r WTP_r DWL_approx_r DWL_exact_r
	
		/*get boostrap replicates for demand and cost curve coefficients*/
		clear 
		use `bs_d'
		gen `alpha_d_r' = _b_cons
		gen `beta_d_r' = _b_r
		keep `alpha_d_r' `beta_d_r'
		save `bs_d', replace
		
		clear
		use `bs_c'
		gen `alpha_c_r' = _b_cons
		gen `beta_c_r' = _b_r
		keep `alpha_c_r' `beta_c_r'
		
		merge 1:1 _n using `bs_d', assert(3) nogen
		
		/*calculate some intermediate quantities for each replicate*/
		gen `alpha_m_r' = `alpha_d_r'*`beta_c_r'/`beta_d_r' + `alpha_c_r'
		gen `beta_m_r'  = 2*`beta_c_r'
		
		gen `phi_c_r' = 1 + `fc_star' - `alpha_c_r' - `beta_c_r'
		gen `psi_c_r' = `phi_c_r'^2 - 4*`alpha_c_r'*`beta_c_r'
		
		gen `phi_m_r' = 1 + `fc_star' - `alpha_m_r' - `beta_m_r'
		gen `psi_m_r' = `phi_m_r'^2 - 4*`alpha_m_r'*`beta_m_r'	
		
		/*calculate efficient and equilibrium outcomes for each replicate*/
		gen `QEF_r' = `alpha_d_r' + (`beta_d_r'*(`phi_m_r' - sqrt(`psi_m_r')))/(2*`beta_m_r')
		gen `rEF_r' = ((`phi_m_r' - sqrt(`psi_m_r')))/(2*`beta_m_r')
		
		gen `QEQ_r' = `alpha_d_r' + (`beta_d_r'*(`phi_c_r' - sqrt(`psi_c_r')))/(2*`beta_c_r')
		gen `rEQ_r' = ((`phi_c_r' - sqrt(`psi_c_r')))/(2*`beta_c_r')
		
		/*approximate welfare loss for each replicate*/		
		gen `MC_r'  = (`alpha_m_r'+`beta_m_r'*`rEQ_r')
		gen `WTP_r' = `rEQ_r'*(1+`fc_star'-`alpha_m_r'-`beta_m_r'*`rEQ_r')
		gen `DWL_approx_r' = 50*(`QEF_r'-`QEQ_r')*(`WTP'-`MC_r')
		
		/*exact welfare loss for each replicate*/		
		gen `DWL_exact_r' = .
		assert _N == `reps'
		forvalues i = 1(1)`reps'{
			
			/*skip replicates that would produce missing welfare estimates*/
			if `psi_m_r'[`i'] < 0 | `psi_c_r'[`i'] < 0{
				local ++i
			}
			
			tempvar x r_d mc wtp fntDemand fntMC
			range `x' `QEQ_r'[`i'] `QEF_r'[`i'] 100
			gen `r_d' = (`x'-`alpha_d_r'[`i'])/`beta_d_r'[`i']
			gen `mc'  = (`alpha_m_r'[`i']+`beta_m_r'[`i']*`r_d')	
			gen `wtp' = (1+`fc_star'-`alpha_m_r'[`i']-`beta_m_r'[`i']*`r_d')*`r_d'
			integ `wtp' `x', gen(`fntDemand')
			local IntDemand = r(integral)
			integ `mc'  `x', gen(`fntMC')
			local IntMC = r(integral)
			replace `DWL_exact_r' = 100*(`IntDemand'-`IntMC') in `i'
			drop `x' `r_d' `mc' `fntDemand' `fntMC'
		}

		/*store confidence intervals for quantites of interest*/
		local ncol = colsof(`b_w')
		local colnames : colnames `b_w'
		matrix `ci_w' = J(2,`ncol',.)
		matrix coln `ci_w' = `colnames'
		matrix rown `ci_w' = ll ul
		
		foreach col of local colnames{
			
			_pctile ``col'_r', p(`ll_pctile' `ul_pctile')
			local ll = r(r1)
			local ul = r(r2)
			
			matrix `ci_w'[1, colnumb(`ci_w', "`col'")] = `ll'
			matrix `ci_w'[2, colnumb(`ci_w', "`col'")] = `ul'
			
		}
		
	restore  
	}
	}
	
	assert `psi_c' > 0
				if _rc != 0 {
					
					display as error "psi_c is negative;"
					display as error `"the fixed cost parameter may be too small"'
					exit 198 
					
				} 
	
	/***************************************************************************
	  concatenate results
	***************************************************************************/	
	
	/*temporary matrix names for storing estimates*/
	tempname b V
	
	/*collect coefficient vectors*/
	matrix `b' = `b_d_iv', `b_d_fs', `b_c_iv', `b_c_fs'
		
	/*collect variance matrices*/
	#delimit ;
	
	mata: st_matrix("`V'",
		blockdiag(st_matrix("`V_d_iv'"),
			blockdiag(st_matrix("`V_d_fs'"),
				blockdiag(st_matrix("`V_c_iv'"),
					st_matrix("`V_c_fs'")
				)
			)
		)
	);
			
	mat coln `V' = 
		`: coln `V_d_iv'' 
		`: coln `V_d_fs''  
		`: coln `V_c_iv'' 
		`: coln `V_c_fs'';
		
	mat rown `V' = 
		`: rown `V_d_iv'' 
		`: rown `V_d_fs''  
		`: rown `V_c_iv''  
		`: rown `V_c_fs'';
		
		
	mat coleq `V' = 
		`: coleq `V_d_iv'' 
		`: coleq `V_d_fs'' 
		`: coleq `V_c_iv'' 
		`: coleq `V_c_fs'';
		
	mat roweq `V' = 
		`: roweq `V_d_iv'' 
		`: roweq `V_d_fs'' 
		`: roweq `V_c_iv'' 
		`: roweq `V_c_fs'';

	#delimit cr
	
	/***************************************************************************
	  return and display output
	***************************************************************************/
	ereturn post `b' `V', esample(`touse')
	ereturn matrix welfare_estimates = `b_w'
	ereturn scalar N_d = `N_d'
	ereturn scalar N_c = `N_c'
	ereturn scalar F_d = `F_d'
	ereturn scalar F_c = `F_c'
	ereturn local cmd "welfare_est"
	ereturn local cmdline `"welfare_est `0'"'

	if "`bs_switch'" == "on"{
		ereturn scalar bootstrap_reps = `reps'
		ereturn scalar welfare_ci_level = `level'
		ereturn matrix welfare_ci = `ci_w'
	}
	
	if "`fcest_switch'" == "on"{
		ereturn matrix fcest_grid_eval = `grid_eval'
		ereturn scalar fc_star = `fc_star'
	}
	
	ereturn display
	
	if "`bs_switch'" == "off"{
		#delimit ;
		
			esttab ., cells("welfare_estimates") 
				title("Implied Quantities") 
				collabels("estimate") 
				nomtitles nonumbers noobs noabbrev; 
		
		#delimit cr
	}
	
	else if "`bs_switch'" == "on"{
		
		#delimit ;
	
			esttab ., cells("welfare_estimates welfare_ci[ll] welfare_ci[ul]")
				title("Implied Quantities")
				collabels("estimate" "    [`level'% Conf." "Interval]    ") 
				nomtitles nonumbers noobs noabbrev; 
		
		#delimit cr
	}
	
	
	/***************************************************************************
	  create figure 
	***************************************************************************/
	if "`figure_switch'" == "on" {
		clear
		set obs 10000
		
		/*range of interest rates to plot*/
		local max_r = 1
		gen r=runiform(0,`max_r')
		sort r
		
		/*demand and cost curves*/ 
		gen D=`alpha_d'+`beta_d'*r
		gen AC=`alpha_c'+`beta_c'*r
		gen AC_tilde = AC/(1 + `fc' - AC)
		gen MC= `alpha_d'*`beta_c'/`beta_d'+`alpha_c'+`beta_c'*r*2
		gen MC_tilde = MC/(1 + `fc' - MC)
		gen WTP = (-(`alpha_d'/`beta_d') + (1/`beta_d')*D)*(1 + `fc' - MC)
		
		/*implied quantities*/
		local phi_c = 1 + `fc' - `alpha_c' - `beta_c'
		local psi_c = `phi_c'^2 - 4*`alpha_c'*`beta_c'
		
		local QEF = `alpha_d'+ (`alpha_d'*`beta_c'+`alpha_c'*`beta_d')/(1-2*`beta_c')
		local rEF = (((`alpha_d'*`beta_c')/`beta_d')+`alpha_c')/(1-2*`beta_c')
		
		local QEQ = `alpha_d' + (`beta_d'*(`phi_c' - sqrt(`psi_c')))/(2*`beta_c')
		local rEQ = ((`phi_c' - sqrt(`psi_c')))/(2*`beta_c')
		
		local MCEQ = `alpha_d'*`beta_c'/`beta_d'+`alpha_c'+`beta_c'*`rEQ'*2
		local WTPEQ = (-(`alpha_d'/`beta_d') + (1/`beta_d')*`QEQ')*(1 + `fc' - `MCEQ')
		
		local MCEF = `alpha_d'*`beta_c'/`beta_d'+`alpha_c'+`beta_c'*`rEF'*2
		local WTPEF = (-(`alpha_d'/`beta_d') + (1/`beta_d')*`QEF')*(1 + `fc' - `MCEF')
		
		/*demand level at max r being plotted*/
		local D_max = `alpha_d' + `beta_d'*`max_r'
		
		#d ;

		/***************************************************************************
			basic plot
		***************************************************************************/
		
		/*panel A*/
		twoway 
			
			/*lines marking equilibrium and efficient outcomes*/
			(func y = `rEQ', range(`D_max' `QEQ') lp(shortdash) lw(thin) lc(gs4))
			(func y = `QEQ', range(0 `rEQ') lp(shortdash) lw(thin) lc(gs4) hor)
			(func y = `rEF', range(`D_max' `QEF') lp(shortdash) lw(thin) lc(gs4))
			(func y = `QEF', range(0 `rEF') lp(shortdash) lw(thin) lc(gs4) hor)
			
			/*demand and cost curves*/
			(line r D, lw(*2) lc(`d_wtp'))
			(line AC_tilde D, lw(*2) lc(`ac_mc') lp(longdash))
			(line MC_tilde D, lw(*2) lc(`ac_mc')),
			
			/*text marking equilibrium and efficient outcomes*/
			text(`rEQ' `QEQ' "(Q{sup:EQ}, r{sup:EQ})", place(ne) margin(tiny) size(*.9))
			text(`rEF' `QEF' "(Q{sup:EF}, r{sup:EF})", place(ne) margin(tiny) size(*.9))	
			
			/*axis titles*/
			ytitle("Price/Cost/WTP", margin(right))
			xtitle("Quantity", margin(top))
			
			/*legend*/
			legend(
				label(5 "Demand Curve")
				label(6 `=ustrunescape("A\u0360\C Curve")') 
				label(7 `=ustrunescape("M\u0360\C Curve")')
				order(5 6 7) cols(1) symxsize(*0.5)
				pos(2) ring(0) region(style(none))
				size(small)
			)
			
			/*overall plot options*/
			yscale(range(0 1))
			ylabel(0(0.2)1)
			xsize(16)
			ysize(9)
			scheme(`scheme') 
			graphregion(margin(r+10))
			plotregion(style(none) margin(vsmall))
			name(empirical_analog_A, replace);
		
		/*panel B*/
		twoway 
			
			/*deadweight loss shading*/
			(rarea WTP MC D if D > `QEQ' & D < `QEF', color(`dwl') lw(none))
			
			/*lines marking WTP at equilibrium and efficient outcomes*/
			(func y = `WTPEQ', range(`D_max' `QEQ') lp(shortdash) lw(thin) lc(gs4))
			(func y = `QEQ', range(0 `WTPEQ') lp(shortdash) lw(thin) lc(gs4) hor)
			(func y = `WTPEF', range(`D_max' `QEF') lp(shortdash) lw(thin) lc(gs4))
			(func y = `QEF', range(0 `WTPEF') lp(shortdash) lw(thin) lc(gs4) hor)
			
			/*WTP and MC curves*/
			/*(line r D, lw(*2) lc(edkblue) lp(dash))*/
			(line WTP D, lw(*2) lc(`d_wtp'))
			(line MC D, lw(*2) lc(`ac_mc')),
			
			/*text marking WTP at equilibrium and efficient outcomes*/
			text(`WTPEQ' `QEQ' "(Q{sup:EQ}, WTP(r{sup:EQ}))", place(ne) margin(tiny) size(*.9))
			text(`WTPEF' `QEF' "(Q{sup:EF}, WTP(r{sup:EF}))", place(ne) margin(tiny) size(*.9))	
			
			/*axis titles*/
			ytitle("Price/Cost/WTP", margin(right))
			xtitle("Quantity", margin(top))
			
			/*legend*/
			legend(
				/*label(6 "Demand Curve")*/
				label(6 "WTP Curve") 
				label(7 "MC Curve")
				order(6 7) cols(1) symxsize(*0.5)
				pos(2) ring(0) region(style(none))
				size(small)
			)
			
			/*overall plot options*/
			yscale(range(0 1))
			ylabel(0(0.2)1)
			xsize(16)
			ysize(9)
			scheme(`scheme') 
			graphregion(margin(r+10))
			plotregion(style(none) margin(vsmall))
			name(empirical_analog_B, replace);
			
		#delimit cr
		
	}
	
	/* re-load user's data*/
	use `user_data', clear 
	
end

/*******************************************************************************
	program to parse instrumental variables 
*******************************************************************************/
program parse_iv, sclass
  
  /***************************************************************************
	  define program syntax
	***************************************************************************/
	syntax varlist(numeric min = 1)
   
  /****************************************************************************
	  parse varlist 
  ****************************************************************************/
   
	tokenize `varlist'
	local iv = * 
	
	/***************************************************************************
	  return parsed options
	***************************************************************************/

	sreturn local iv `iv'
	
end


/*******************************************************************************
  program to parse welfare loss confidence interval options
*******************************************************************************/
program parse_wcis_opt, sclass
  
  /***************************************************************************
	  define program syntax
	***************************************************************************/
  #delimit ; 
	
	syntax [anything(name=switch)] 
		[,
			level(cilevel)          /*confidence interval level*/
			seed(numlist max=1)     /*random number seed*/
			reps(integer 50)        /*number of bootstrap replicates*/
		];
	
	#delimit cr
	
	/***************************************************************************
	  parse options
	***************************************************************************/
	
	/*make sure first argument is valid*/
	capture assert inlist("`switch'", "", "on", "off")
	if _rc != 0{
		
		display as error "invalid wcis argument;"
		display as error `"only "on" or "off" is allowed"'
		exit 198
		
	}
	
	/*default value for first argument is on*/
	if "`switch'" == ""{
		local switch = "on"
	}
	
	/*default seed value = current time*/
	if "`seed'" == ""{
		local seed = clock(c(current_time), "hms" )
	}
	
	/***************************************************************************
	  return parsed options
	***************************************************************************/
	sreturn local wcis_switch `switch'
	sreturn local wcis_level `level'
	sreturn local wcis_seed `seed'
	sreturn local wcis_reps `reps'
end	


/*******************************************************************************
	program to parse fixed cost options
*******************************************************************************/
program parse_fcest_opt, sclass
  
  /***************************************************************************
	  define program syntax
	***************************************************************************/
    #delimit ; 
	
	syntax [anything(name=switch)] 
		[,
			fc(real 0)               /*fixed cost input if not estimating*/
			grid(numlist ascending)  /*grid over which to search*/
			rtarget(numlist max=1)   /*target interest rate to match*/
			qtarget(numlist max=1)   /*target quantity to match*/
		];
	
	#delimit cr
	
	/***************************************************************************
	  parse options
	***************************************************************************/
	
	/*make sure first argument is valid*/
	capture assert inlist("`switch'", "", "on", "off")
	if _rc != 0{
		
		display as error "invalid fcest argument;"
		display as error `"only "on" or "off" is allowed"'
		exit 198
		
	}
	
	/*default value for first argument is off*/
	if "`switch'" == ""{
		local switch = "off"
	}
	
	/*grid option must be non-empty if estimating fixed cost*/
	if "`switch'" == "on"{
		
		capture assert !missing("`grid'")
		if _rc != 0{
			
			display as error "invalid grid option;"
			display as error `"must specify an ascending numlist"'
			exit 198
			
		}
		
	}
	
	/*target values must be non-empty if estimating fixed cost*/
	if "`switch'" == "on"{
		
		capture assert !missing("`rtarget'") & !missing("`qtarget'")
		if _rc != 0{
			
			display as error "invalid rtarget or qtarget option;"
			display as error `"must speficy an interest rate and quantity to target"'
			exit 198
			
		}
		
	}
	
	/***************************************************************************
	  return parsed options
	***************************************************************************/
	sreturn local fcest_switch `switch'
	sreturn local fcest_fc_input  `fc'
	sreturn local fcest_grid `grid'
	sreturn local fcest_rtarget `rtarget'
	sreturn local fcest_qtarget `qtarget'
	
end


/*******************************************************************************
	program to parse figure options
*******************************************************************************/
program parse_figure_opt, sclass
  
  /***************************************************************************
	  define program syntax
	***************************************************************************/
    #delimit ; 
	
	syntax [anything(name=switch)]
		[,
			scheme(string)
			d_wtp(string)
			ac_mc(string)
			dwl(string)
		];
	
	#delimit cr
	
	/***************************************************************************
	  parse options
	***************************************************************************/
	
	/*make sure first argument is valid*/
	capture assert inlist("`switch'", "", "on", "off")
	if _rc != 0{
		
		display as error "invalid figure argument;"
		display as error `"only "on" or "off" is allowed"'
		exit 198
		
	}
	
	/*default value for first argument is off*/
	if "`switch'" == ""{
		local switch = "off"
	}
	
	/*default values if user didn't choose scheme and colors*/ 
	if "`scheme'" == "" {
		local scheme = "s1mono"
	}
	if "`d_wtp'" == "" {
		local d_wtp = "edkblue"
	}
	if "`ac_mc'" == "" {
		local ac_mc = "orange_red"
	}
	if "`dwl'" == "" {
		local dwl = "gs12"
	}
	
	
	/***************************************************************************
	  return parsed options
	***************************************************************************/
	sreturn local figure_switch `switch'
	sreturn local figure_scheme `scheme'
	sreturn local figure_d_wtp `d_wtp'
	sreturn local figure_ac_mc `ac_mc'
	sreturn local figure_dwl `dwl' 

	
end


